import React, { FunctionComponent, useState, useEffect } from 'react';
import { useRouter } from 'next/router';

import { Card, LoadingSpinner, GridContainer, GridRow, GridColumn } from '../containers';
import { Input, Button, DropdownButton, IDropdownMenuItem, Select, ISelectOption } from '../input';
import { Separator } from '../text';

import { IAddress } from '@felixbaum/me.easymove.common/dist/interfaces/addresses';

import theme from '../../styles/theme';
import core from '@felixbaum/me.easymove.core';


type IAddressDialogProp = {
    isNew: boolean;
    addressLoading?: boolean;
    saveLoading?: boolean;
    currentAddress: IAddress;
    style?: React.CSSProperties;

    onSave?: (address: IAddress) => void;
}

const AddressDialog: FunctionComponent<IAddressDialogProp> = ({ isNew, addressLoading, saveLoading, currentAddress, style, onSave }) => {
    const router = useRouter();
    const [address, setAddress] = useState<IAddress>();
    
    useEffect(() => { 
        setAddress(currentAddress);
    }, [currentAddress]);


    const onAddressPropertyChange = (name: string, value: string) => {
        setAddress({ ...address, [name]: value });
    }

    const onAddressPayloadPropertyChange = (name: string, value) => {
        setAddress({ ...address, payload: { ...address.payload, [name]: value }});
    }


    const onSaveButtonClick = (event: React.MouseEvent) => {
        event.preventDefault();

        onSave(address);
    }

    const onAddressDeleteClick = (event: React.MouseEvent) => {
        event.preventDefault();

        core.address.deleteAddress(address.addressId).then(result => {
            router.push('/user/addresses');
        }).catch(error => {
            
        });
    }

    const addressActions: Array<IDropdownMenuItem> = [{ text: 'Löschen', onClick: onAddressDeleteClick, backgroundColor: theme.colors.error, textColor: theme.colors.errorDark }];

    return (
        <Card style={style}>

            {addressLoading &&
                <LoadingSpinner style={{ marginLeft: 'auto', marginRight: 'auto' }} color={theme.colors.primary}/>
            }

            {!addressLoading && 
            <GridContainer>
                <GridRow style={{ marginBottom: '30px' }}>
                    <GridColumn position="left">
                        <Input description="Bezeichnung der Adresse" name="description" value={address !== undefined ? address.description : ''} onValueChange={onAddressPropertyChange}/>
                    </GridColumn>

                    <GridColumn position="right" style={{ display: 'flex' }}>
                        <div style={{ flexBasis: '70%', paddingRight: '15px' }}>
                        </div>

                        <div style={{ flexBasis: '30%' }}>
                            <DropdownButton style={{ marginTop: '36px' }} label="Verwalten" menuItems={addressActions}/>
                        </div>
                    </GridColumn>
                </GridRow>

                <GridRow>
                    <Separator text="Basisdaten"/>
                </GridRow>

                <GridRow style={{ marginTop: '30px' }}>
                    <GridColumn position="left">
                        <Select description="Anrede" name="salutation" options={[ { text: 'Frau', value: 'Frau' }, { text: 'Herr', value: 'Herr' } ]} value={address !== undefined ? address.payload.salutation : ''} onValueChange={onAddressPayloadPropertyChange}/>
                    </GridColumn>

                    <GridColumn position="right">
                        <Input description="Titel" name="title" value={address !== undefined ? address.payload.title : ''} onValueChange={onAddressPayloadPropertyChange}/>
                    </GridColumn>
                </GridRow>

                <GridRow style={{ marginTop: '15px' }}>
                    <GridColumn position="left">
                        <Input description="Vorname" name="firstname" value={address !== undefined ? address.payload.firstname : ''} onValueChange={onAddressPayloadPropertyChange}/>
                    </GridColumn>

                    <GridColumn position="right">
                        <Input description="Nachname" name="lastname" value={address !== undefined ? address.payload.lastname : ''} onValueChange={onAddressPayloadPropertyChange}/>
                    </GridColumn>
                </GridRow>

                <GridRow style={{ marginTop: '15px' }}>
                    <GridColumn position="left" style={{ display: 'flex' }}>
                        <div style={{ flexBasis: '70%', paddingRight: '15px' }}>
                            <Input description="Straße" name="street" value={address !== undefined ? address.payload.street : ''} onValueChange={onAddressPayloadPropertyChange}/>
                        </div>

                        <div style={{ flexBasis: '30%' }}>
                            <Input description="Hausnummer" name="houseno" value={address !== undefined ? address.payload.houseno : ''} onValueChange={onAddressPayloadPropertyChange}/>
                        </div>
                    </GridColumn>

                    <GridColumn position="right" style={{ display: 'flex' }}>
                        <div style={{ flexBasis: '30%', paddingRight: '15px' }}>
                            <Input description="Postleitzahl" name="zip" value={address !== undefined ? address.payload.zip : ''} onValueChange={onAddressPayloadPropertyChange}/>
                        </div>

                        <div style={{ flexBasis: '70%' }}>
                            <Input description="Ort" name="city" value={address !== undefined ? address.payload.city : ''} onValueChange={onAddressPayloadPropertyChange}/>
                        </div>
                    </GridColumn>
                </GridRow>

                <GridRow style={{ marginTop: '15px' }}>
                    <GridColumn position="left">
                        
                    </GridColumn>

                    <GridColumn position="right">
                        <Input description="Adressinfo" name="info" value={address !== undefined ? address.payload.info : ''} onValueChange={onAddressPayloadPropertyChange}/>
                    </GridColumn>
                </GridRow>

                <GridRow style={{ marginTop: '30px' }}>
                    <GridColumn position="left">
                        
                    </GridColumn>

                    <GridColumn position="right">
                        <Button text="Speichern" loading={saveLoading} onClick={onSaveButtonClick}/>
                    </GridColumn>
                </GridRow>
            </GridContainer>
            }
        </Card>
    )
}

export default AddressDialog;