import React, { useState, useEffect } from 'react';
import { GetServerSideProps } from 'next';
import { useRouter } from 'next/router';
import { useToasts } from 'react-toast-notifications';

import { protectedByLogin } from '../../../utils/protection';

import { LayoutFuntionComponent } from '../../_app';

import { CenteringContainer, SiteWidthContainer, FlexColumnContainer, FlexColumn, Card, LoadingSpinner } from '../../../components/containers';
import { Heading, Label } from '../../../components/text';
import { Button } from '../../../components/input';
import UserLayout from '../../../components/layouts/UserLayout';
import theme from '../../../styles/theme';

import core from '@felixbaum/me.easymove.core';
import { UITab } from '@felixbaum/me.easymove.common/dist/enums/ui';


const Addresses: LayoutFuntionComponent = () => {
    const [loading, setLoading] = useState(true);
    const [addresses, setAddresses] = useState<Array<{ addressId: number, description: string, changedOn: Date, subscriberCount: number }>>([]);
    const router = useRouter();
    const { addToast } = useToasts();

    core.ui.state.CURRENT_TAB.set(UITab.USER_ADDRESSES);

    useEffect(() => {
        core.address.actions.getAddressesMetadata().then(result => {
            setLoading(false);

            if (result.code === 200) {
                result.result.forEach(address => {
                    address.changedOn = new Date(address.changedOn);
                });
                setAddresses(result.result);
            }
        }).catch(error => {
            console.log(error);
            setLoading(false);
            addToast('Ups! Da ist etwas schief gelaufen!', {
                appearance: 'error',
                autoDismiss: true
            });
        })
    }, []);

    const newAddress = (event: React.MouseEvent) => {
        event.preventDefault();

        router.push('/user/addresses/new');
    }

    const openAddress = (addressId: number) => {
        router.push('/user/addresses/' + addressId);
    }

    return (
        <SiteWidthContainer>
            <FlexColumnContainer>
                <FlexColumn>
                    <Heading fontColor="white" fontSize={theme.fontSizes.big}>Adressen</Heading>
                </FlexColumn>

                <FlexColumn>
                    <Button text="Neue Adresse" onClick={newAddress}/>
                </FlexColumn>
            </FlexColumnContainer>

            {loading &&
                <CenteringContainer>
                    <LoadingSpinner style={{ marginLeft: 'auto', marginRight: 'auto' }}/>
                </CenteringContainer>
            }

            <div style={{ display: 'flex', flexWrap: 'wrap', justifyContent: 'space-between', rowGap: '50px', marginTop: '100px' }}>
                {
                    addresses.map((address, index) => {
                        return (
                            <Card key={address.addressId} onClick={(event: React.MouseEvent) => { event.preventDefault(); openAddress(address.addressId); }} style={{ flex: '0 0 28%', padding: '20px', textAlign: 'start', justifyContent: 'flex-start', alignItems: 'normal' }}>
                                <Heading fontColor={theme.colors.primary} fontSize={23}>{address.description}</Heading>

                                <Label style={{ marginTop: '10px' }} description="Letzte Änderung" value={`${address.changedOn.getUTCDate()}.${address.changedOn.getMonth()}.${address.changedOn.getFullYear()} - ${address.changedOn.getHours()}:${address.changedOn.getMinutes()}`}/>
                                
                                <Label style={{ marginTop: '10px' }} description="Abonnenten" value={address.subscriberCount.toString()}/>
                            </Card>
                        )
                    })
                }
            </div>
        </SiteWidthContainer>
    )
}

Addresses.Layout = UserLayout;

export const getServerSideProps: GetServerSideProps = protectedByLogin;

export default Addresses;