import React, { FunctionComponent, useState, useEffect } from 'react';
import { useRouter } from 'next/router';
import { GetServerSideProps } from 'next';
import { useToasts } from 'react-toast-notifications';
import { setCookie, parseCookies } from 'nookies';

import { alreadyLoggedIn } from '../../utils/protection';

import { FullPageContainer, CenteringContainer, Card } from '../../components/containers';
import { Heading, Link } from '../../components/text';
import { Input, Button } from '../../components/input';

import theme from '../../styles/theme';
import { isNotEmpty, isValidEmail } from '../../utils/inputvalidator';
import core from '@felixbaum/me.easymove.core';


const Login: FunctionComponent = () => {
    const router = useRouter();
    const [loading, setLoading] = useState(false);
    const [email, setEmail] = useState("");
    const [emailError, setEmailError] = useState(""); 
    const [password, setPassword] = useState("");
    const [passwordError, setPasswordError] = useState("");
    const { addToast } = useToasts();

    const onSubmit = (event: React.MouseEvent) => {
        event.preventDefault();

        setEmailError("");
        setPasswordError("");

        let valid = true;
        if (!isNotEmpty(email, 'E-Mail-Adresse', setEmailError) || !isValidEmail(email, setEmailError)) valid = false;
        if (!isNotEmpty(password, 'Passwort', setPasswordError)) valid = false;
        if (!valid) return;
        
        setLoading(true);

        core.authentication.actions.login({ email: email, password: password }).then((result) => {
            setLoading(false);

            if (!result) {
                addToast('Ups! Da ist etwas schief gelaufen!', {
                    appearance: 'error',
                    autoDismiss: true
                });
            } else if (result.code === 200) {
                setCookie(null, '__EM_SESSION_TOKEN', core.authentication.state.SESSION_TOKEN.value.toString(), {
                    maxAge: 24 * 60 * 60,
                    path: '/',
                    secure: process.env.NEXT_PUBLIC_NODE_ENV !== 'development'
                });

                router.push('/user/addresses');
            } else if (result.code === 401) {
                setEmailError('Falsche E-Mail oder Passwort');
                setPasswordError('Falsche E-Mail oder Passwort');
            }
        }).catch(error => {
            setLoading(false);
            addToast('Ups! Da ist etwas schief gelaufen!', {
                appearance: 'error',
                autoDismiss: true
            });
        });
    }

    return (
        <>

        <style jsx global>{`
            #nonresponsive-container {
                display: none;
            }

            #responsive-container {
                display: block;
            }

            @media only screen and (min-width: 800px) {
                #nonresponsive-container {
                    display: flex;
                }

                #responsive-container {
                    display: none;
                }
            }
        `}</style>


        <FullPageContainer background={theme.colors.gradientBackground}>
            <CenteringContainer id="nonresponsive-container">
                <Card style={{ width: '700px' }}>
                    <Heading fontSize={theme.fontSizes.fat} fontColor={theme.colors.primary}>EasyMove</Heading>

                    <form noValidate style={{ width: '80%', marginTop: '50px' }}>
                        <Input name="email" description="E-Mail-Adresse" type="email" error={emailError} onValueChange={ (name, value) => setEmail(value) }/>

                        <Input name="password" description="Passwort" type="password" style={{ marginTop: '20px' }} onValueChange={ (name, value) => setPassword(value) } error={passwordError}/>

                        <div style={{ display: 'flex', textAlign: 'center', alignItems: 'stretch', justifyContent: 'flex-end' }}>
                            <Link style={{ textAlign: 'end' }} href="/user/reset">Passwort vergessen?</Link>
                        </div>

                        <Button style={{ marginTop: '50px' }} type="submit" text="Anmelden" loading={loading} onClick={onSubmit}/>
                    </form>
                </Card>

                <p style={{ color: 'white', margin: 0, marginTop: '5px' }}>Du hast kein Konto? <Link color="black" href="/user/register">Jetzt registrieren.</Link> </p>
            </CenteringContainer>
        </FullPageContainer>

        </>
    );
}

export const getServerSideProps: GetServerSideProps = alreadyLoggedIn;

export default Login;