import React, { FunctionComponent, useState } from 'react';
import { GetServerSideProps } from 'next';
import { useRouter } from 'next/router';
import { useToasts } from 'react-toast-notifications';

import { alreadyLoggedIn } from '../../utils/protection';

import { FullPageContainer, CenteringContainer, Card } from '../../components/containers';
import { Heading, Link } from '../../components/text';
import { Input, Button } from '../../components/input';

import { isNotEmpty, isValidEmail, isValidPassword } from '../../utils/inputvalidator';
import core from '@felixbaum/me.easymove.core';
import theme from '../../styles/theme';


const Register: FunctionComponent = () => {
    const router = useRouter();
    const [loading, setLoading] = useState(false);
    const { addToast } = useToasts();

    const [email, setEmail] = useState("");
    const [emailError, setEmailError] = useState("");

    const [firstname, setFirstname] = useState("");
    const [firstnameError, setFirstnameError] = useState("");

    const [lastname, setLastname] = useState("");
    const [lastnameError, setLastnameError] = useState("");

    const [password, setPassword] = useState("");
    const [passwordError, setPasswordError] = useState("");


    const onSubmit = (event: React.MouseEvent) => {
        event.preventDefault();

        setEmailError("");
        setFirstnameError("");
        setLastnameError("");
        setPasswordError("");

        let valid = true;
        if (!isNotEmpty(email, 'E-Mail-Adresse', setEmailError) || !isValidEmail(email, setEmailError)) valid = false;
        if (!isNotEmpty(firstname, 'Vorname', setFirstnameError)) valid = false;
        if (!isNotEmpty(lastname, 'Nachname', setLastnameError)) valid = false;
        if (!isNotEmpty(password, 'Passwort', setPasswordError) || !isValidPassword(password, setPasswordError)) valid = false;
        if (!valid) return;

        setLoading(true);

        core.authentication.actions.register({ email: email, firstname: firstname, lastname: lastname, password: password }).then((result) => {
            setLoading(false);
            
            if (!result) {
                addToast('Ups! Da ist etwas schief gelaufen!', {
                    appearance: 'error',
                    autoDismiss: true
                });
            } else if (result.code === 200) {
                router.push('/user/welcome');
            } else if (result.code === 109) {
                setEmailError('E-Mail-Adresse bereits registriert');
            } else if (result.code === 111) {
                setPasswordError('Passwort zu kurz');
            } else if (result.code === 112) {
                setPasswordError('Passwort zu schwach');
            }
        }).catch((error) => {
            setLoading(false);
            addToast('Ups! Da ist etwas schief gelaufen!', {
                appearance: 'error',
                autoDismiss: true
            });
        });
    }

    return (
        <FullPageContainer background={theme.colors.gradientBackground}>
            <CenteringContainer>
                <Card style={{ width: '700px' }}>
                    <Heading fontSize={theme.fontSizes.fat} fontColor={theme.colors.primary}>EasyMove</Heading>

                    <form noValidate style={{ width: '80%', marginTop: '50px' }}>
                        <Input name="email" description="E-Mail-Adresse" type="email" error={emailError} onValueChange={ (name, value) => setEmail(value) }/>

                        <Input name="firstname" description="Vorname" type="text" style={{ marginTop: '20px' }} error={firstnameError} onValueChange={ (name, value) => setFirstname(value) }/>

                        <Input name="lastname" description="Nachname" type="text" style={{ marginTop: '20px' }} error={lastnameError} onValueChange={ (name, value) => setLastname(value) }/>

                        <Input name="password" description="Passwort" type="password" style={{ marginTop: '20px' }} error={passwordError} onValueChange={ (name, value) => setPassword(value) }/>

                        <Button style={{ marginTop: '50px' }} type="submit" text="Registrieren" loading={loading} onClick={onSubmit}/>
                    </form>
                </Card>

                <p style={{ color: 'white', margin: 0, marginTop: '5px' }}>Du hast schon ein Konto? <Link color="black" href="/user/login">Anmelden.</Link> </p>
            </CenteringContainer>
        </FullPageContainer>
    );
}

export const getServerSideProps: GetServerSideProps = alreadyLoggedIn;

export default Register;